package application;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Random;
import java.util.Scanner;

public class IA extends Joueur {

	private int difficulte;
	private Arbre arbre;
	private int nbTour;
	
	private ArrayList<Coup> historique;
 	
	
	public IA(int difficulte) {
		super();
		this.difficulte = difficulte;
		this.arbre = null;
		this.nbTour = 0;
		this.historique = new ArrayList<Coup>();
	}
	
	public IA(int difficulte, ArrayList<Coup> historique, int nbTour) { 	// Utile pour charger une partie
		super();
		this.difficulte = difficulte;
		this.historique = historique;
		this.nbTour = nbTour;
		
		this.arbre = null;
		initialiserArbre();
	}
	

	public int getDifficulte() {
		return difficulte;
	}
	public Arbre getArbre() {
		return arbre;
	}
	public int getNbTour() {
		return nbTour;
	}
	public boolean getType() {return false;}
	
	private void initialiserArbre () {		//Genere les n prochaines generations de feuilles selon la difficulté

		arbre = new Arbre();

		for (int i=0; i<difficulte; i++) {
			creerNouvGen (arbre, historique);
		}
	}
		
	private void creerNouvGen (Arbre noeud, ArrayList<Coup> acc) {
		
		if (noeud.coupGagnant())
			return;
		
		if (!noeud.getFils().isEmpty()) {
			
			ArrayList<Coup> prochainsCoups = new ArrayList<>();
			
			for (int i = 0; i < acc.size(); i++) {
				prochainsCoups.add(acc.get(i));
			}
			
			if (noeud.getCoup() != null)
				prochainsCoups.add(noeud.getCoup());
			
		
			for (Arbre fils : noeud.getFils()) {
				
				if (! fils.coupGagnant())
					creerNouvGen(fils, prochainsCoups);

			}
			
			return;
		}
		
		for (Coup coup : coupPossible(acc)) {
			
			noeud.ajouterFils(new Arbre(coup));
			
			if (victoireFuture(acc, coup))
				noeud.estGagnant();
			
		}
		
		//acc.clear();
			
	}

	public static boolean dejaPris (ArrayList<Coup> coupsPrecedents, int x, int y) {
		
		for (Coup coup : coupsPrecedents) {
			
			if (coup != null && coup.getX() == x && coup.getY() == y)
				return true;
		}
		return false;
	}

	
	private static ArrayList<Coup> coupPossible (ArrayList<Coup> coupsPrecedents) {

		ArrayList<Coup> coups = new ArrayList<Coup>();
		
		for (int x = 0; x < 4; x++) {
			
			for (int y = 0; y < 4; y++) {
				
				if (! dejaPris(coupsPrecedents, x, y)) {
				
					for (Jeton jeton : EnsJetonDisponible(coupsPrecedents)) { // Ensemble de tout les Jetons		
							
						coups.add(new Coup(x,y, jeton));					
						
					}
				}
			}
		}
		
		return coups;
	}

	private static ArrayList<Jeton> EnsJetonDisponible(ArrayList<Coup> coupsPrecedents) {

		ArrayList<Jeton> tab = new ArrayList<>(Arrays.asList(JetonMenu.tableauDeJeton));  // JetonMenu.tableauDeJeton.clone(); 

		for (Coup c : coupsPrecedents) {
			
			if (c != null && tab.contains(c.getJetonSelect()))
				tab.remove(c.getJetonSelect());

		}

		return tab;

	}
	
	private Coup coupAlea () {						//Choisit un couple aleatoire (pour les premiers tours)
		
		Random rand = new Random();
		
		ArrayList<Coup> coupsP = coupPossible(historique);
		int nbAlea = rand.nextInt(coupsP.size());
		
		return coupsP.get(nbAlea);
	}
	
	private int algoMinMax (Arbre noeud, boolean tourIA) {
		
		if (noeud.getFils() == null || noeud.getFils().isEmpty()) {
			return noeud.coupGagnant() ? (tourIA ? 100 : -100) : 0;
		}
		
		int tmp;
		if (noeud.getFils().get(0).getFils().isEmpty() || noeud.getFils().get(0).getFils() == null) {
			
			for (Arbre fils : noeud.getFils()) {
				
				tmp = algoMinMax(fils, !tourIA);				
				noeud.setVal(noeud.getVal() + tmp);
				
			}
			
			return noeud.getVal();
			
		}

		
		if (!tourIA) {
						
			for (Arbre fils : noeud.getFils()) {
				
				tmp = algoMinMax(fils, true);				
			}
			
			noeud.setVal(noeud.getFils().get(0).getVal()) ;
			for (Arbre fils : noeud.getFils()) {
				
				tmp = fils.getVal();
				
				if (noeud.getVal()>tmp) {
					noeud.setVal(tmp);
				}
			}
			
		}
		
		else {
			
			for (Arbre fils : noeud.getFils()) {
				
				tmp = algoMinMax(fils, false);				
			}
			
			noeud.setVal(noeud.getFils().get(0).getVal()) ;
			for (Arbre fils : noeud.getFils()) {
				
				tmp = fils.getVal();
				
				if (noeud.getVal()<tmp) {
					noeud.setVal(tmp);
				}
			}
			
		}
		
		return noeud.getVal();
	}
	
	private Coup meilleurCoup() {
		
		Arbre tmp = arbre.getFils().get(0);
		
		System.out.println(arbre.getVal());
		for (Arbre noeud : arbre.getFils()) {
			System.out.println(noeud.getVal());
		}
		
		
		for (Arbre noeud : arbre.getFils()) {
			
			if (victoireFuture(historique, noeud.getCoup()))
				return noeud.getCoup();
			
			if (noeud.getVal() > tmp.getVal())
					tmp = noeud;
			
		}
		
		return tmp.getCoup();
	}
	
	private static boolean victoireFuture (ArrayList<Coup> coupPrec, Coup coup) {
		
		Jeton grille[][] = new Jeton[4][4];
		Jeton pieceTmp = coupPrec.get(0).getJetonSelect();
		
		for (int i=1; i<coupPrec.size(); i++) {
			
			grille[coupPrec.get(i).getX()][coupPrec.get(i).getY()] = pieceTmp;
			pieceTmp = coupPrec.get(i).getJetonSelect();
			
		}
		
		grille[coup.getX()][coup.getY()] = pieceTmp;

		for (int x=0; x<4; x++) {
			if (caracCommun(grille[x]))
				return true;
		}
		for (int y=0; y<4; y++) {
			if (caracCommun(new Jeton[] {grille[0][y],grille[1][y],grille[2][y],grille[3][y]}))
				return true;
		}

		if (caracCommun(new Jeton[] {grille[0][0],grille[1][1],grille[2][2],grille[3][3]}))
			return true;

		if (caracCommun(new Jeton[] {grille[0][3],grille[1][2],grille[2][1],grille[3][0]}))
			return true;
		
		
		return false;
	}

	private static boolean caracCommun (Jeton tab[]) {

		boolean test = true;
		
		for (int i=0; i<4; i++) {
			if (tab[i] == null) 
				return false;
		}
		
		for (int i=0; i<3; i++) {
			if (tab[i].getForme() != tab[i+1].getForme())
				test = false;
		}
		if (test) 
			return test;
		
		test = true;
		for (int i=0; i<3; i++) {
			if (tab[i].getCouleur() != tab[i+1].getCouleur())
				test = false;
		}
		if (test) 
			return test;

		test = true;
		for (int i=0; i<3; i++) {
			if (tab[i].getTaille() != tab[i+1].getTaille())
				test = false;
		}
		if (test) 
			return test;

		test = true;
		for (int i=0; i<3; i++) {
			if (tab[i].getPleine() != tab[i+1].getPleine())
				test = false;
		}
		if (test) 
			return test;

		return false;

	}

	
	@Override
	public Coup jouerTour(Coup c) {	
		
		if (nbTour >= 16) {
			return null;
		}
		
		nbTour++;
		historique.add(c);
		Coup tmp;
		
		if (nbTour < 4) {
			tmp = coupAlea();
		}
		
		
		else {
			
			if (nbTour == 5 || nbTour == 4) {
				initialiserArbre();
			}
			else {				
				arbre = Arbre.getFils(arbre, c);		
				creerNouvGen(arbre, historique);
			}
			
			algoMinMax(arbre, true);
			tmp = meilleurCoup();
			
			arbre = Arbre.getFils(arbre, tmp);
			creerNouvGen(arbre, historique);
			
		}
		
		historique.add(tmp);
		nbTour++;
		
		return tmp;
	}
	
	public Coup premierTour () {
		
		Random rand = new Random();
		int nbAlea = rand.nextInt(JetonMenu.tableauDeJeton.length);
		
		return new Coup(-1, -1, JetonMenu.tableauDeJeton[nbAlea]);
	}
	
/*	public static void main(String argv[]) {
		
		
		int grille[][] = new int[4][4];
		
		for (int i=0; i<4; i++) {
			for (int j=0; j<4; j++) {
				
				grille[i][j] = 0;
				
			}
		}
		
		IA Joueur = new IA(4);
		
		int x;
		int y;
		int index;
		
		Coup tmp = null;
		
		Scanner sc = new Scanner(System.in);
		
		do {
			
			for (Jeton j : IA.EnsJetonDisponible(Joueur.historique)) {
				System.out.println(j);
			}
			System.out.println(IA.EnsJetonDisponible(Joueur.historique).size());
			
			
			System.out.println("Selectionner Coup : ");
			x = sc.nextInt();
			y = sc.nextInt();
			index = sc.nextInt();
			
			if (tmp != null)
				grille[x][y] = tmp.getJetonSelect().hashCode();
			
			tmp = Joueur.jouerTour(new Coup(x,y,EnsJetonDisponible(Joueur.historique).get(index)));
			
			System.out.println(tmp.getX());
			System.out.println(tmp.getY());
			System.out.println(tmp.getJetonSelect());
			System.out.println("\n");
			
			for (int i=0; i<4; i++) {
				for (int j=0; j<4; j++) {
					
					System.out.print(grille[i][j]+" ");
					
				}
				
				System.out.print("\n");
			}
			
			
		}
		while (!victoireFuture(Joueur.historique, tmp));

		sc.close();
	} */
	
}
