package application;

import javafx.scene.shape.Rectangle;
import javafx.scene.text.Font;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.control.RadioButton;
import javafx.scene.control.ToggleGroup;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.paint.Color;

public class JetonMenu {
	
	/* --------------------------------------------------- Attributs ---------------------------------------------------------*/
	/*------------------------------------------------------------------------------------------------------------------------*/
	private static int compteurEvent;
	private Rectangle menuJB; //Menu jetons blancs
	private Rectangle menuJN; //Menu jetons noirs
	private Rectangle menu3D; //Affichage 3D des jetons
	private ImageView im3D;
	
	public static final Jeton[] tableauDeJeton = initTabJeton();
	private RadioButton[] tableauBouton;
	private Image[] tableauImage;
	private ImageView[] tableauImageView;
	private Label[] msgBox;
	
	private Button pause;
	private Button restart;
	private Button stopSave;
	private Button exit;
	private Button menuPrinc;

	private int coordX;
	private int coordY;
	private FenetrePartie partie;
	
	/* --------------------------------------------------- Accesseurs --------------------------------------------------------*/
	/*------------------------------------------------------------------------------------------------------------------------*/
	
	//Accesseurs lecture
	public Rectangle getMenuJB() { return menuJB;}
	public Rectangle getMenuJN() { return menuJN;}
	public Rectangle getMenu3D() { return menu3D;}
	public ImageView getIm3D() { return im3D;}

	public int getCoordX() {return coordX;}
	public int getCoordY() {return coordY;}
	public FenetrePartie getPartie() {return partie;}
	
	public Button getPause() { return pause;}
	public Button getRestart() { return restart;}
	public Button getStopSave() { return stopSave;}
	public Button getExit() { return exit;}
	public Button getMenuPrinc() { return menuPrinc;}
	
	public static Jeton getTableauDeJeton(int i) { return tableauDeJeton[i]; }
	public RadioButton getTableauBouton(int i) { return tableauBouton[i];}
	public Image getTableauImage(int i) { return tableauImage[i];}
	public ImageView getTableauImageView(int i) { return tableauImageView[i];}
	public Label getMsgBox(int i) {return msgBox[i];}
	
	//Accesseurs ecriture
	public void setX(int x) {
		coordX = x;
	}
	public void setY(int y) {
		coordY = y;
	}	
	public void setPartie(FenetrePartie partie) {
		this.partie = partie;
	}
	public void setMenuJB(float a, float b, float c, float d) {
		this.menuJB = new Rectangle(a,b,c,d);
	}
	public void setMenuJN(float a, float b, float c, float d) {
		this.menuJN = new Rectangle(a,b,c,d);
	}
	public void setMenu3D(float a, float b, float c, float d) {
		this.menu3D = new Rectangle(a,b,c,d);
	}
	public void setIm3D(ImageView im) {
		this.im3D = im;
	}
	public void removeIm3D() { this.im3D.setImage(null); }
	
	public void setTableauDeJeton(int i, Jeton j) {tableauDeJeton[i] = j;}
	public void setTableauBouton(int i, RadioButton r) {this.tableauBouton[i] = r;}
	public void setTableauImage(int x, Image i) {tableauImage[x] = i;}
	public void setTableauImageView(int x, ImageView i) {this.tableauImageView[x] = i;}
	public void setMsgBox(int i, Label l) {this.msgBox[i] = l;}
	
	public void setPause(String s) {
		this.pause = new Button("s");
	}
	public void setRestart(String s) {
		this.restart = new Button("s");
	}
	public void setStopSave(String s) {
		this.stopSave = new Button("s");
	}
	public void setExit(String s) {
		this.exit = new Button("s");
	}
	public void setMenuPrinc(String s) {
		this.menuPrinc = new Button("s");
	}
	
	
	/* -------------------------------------------------- CONSTRUCTEUR -------------------------------------------------------*/
	/*------------------------------------------------------------------------------------------------------------------------*/
	public JetonMenu (Rectangle jb, Rectangle jn, Rectangle d, FenetrePartie partie) {
		menuJB = jb;
		menuJN = jn;
		menu3D = d;
		im3D = new ImageView();
		compteurEvent=0;
		
	
		this.partie = partie;
		coordX = -1;
		coordY = -1;
		
		tableauBouton = new RadioButton[16];
		
		tableauImageView = new ImageView[16];
		
		//Label pour afficher les messages dans la box en bas à droite
		msgBox = new Label[6];
		msgBox[0] = new Label("Joueur 1 choisit une piece");
		msgBox[1] = new Label("Joueur 2 joue la piece choisie");
		msgBox[2] = new Label("Joueur 2 choisit une piece");
		msgBox[3] = new Label("Joueur 1 joue la piece choisie");
		msgBox[4] = new Label("Victoire du joueur 1");
		msgBox[5] = new Label("Victoire du joueur 2");
		
		//différents boutons du menu en jeu
		pause = new Button("Pause");
		restart = new Button("Restart");
		stopSave = new Button("Stop & Save");
		exit = new Button("Exit");
		menuPrinc = new Button("Menu Principal");
		
		
		
	}


	private static Jeton[] initTabJeton () {
		
		Jeton[] tab = new Jeton[16];
		for(int i=0; i<16; i++) {
			tab[i] = new Jeton();
		}
		
		String image = "";
		for(int i=0; i<16; i++) {
			
			
			
			image = "Images/jeton2D/"+(i+1)+".png";
			tab[i].setIm(image);
			
			//tableauDeJeton[i].setInit(true);
			if(i<8) {tab[i].setCouleur(true);}
			if(i%2 == 0) {tab[i].setTaille(true);}
			if(i<4 || (i>=8 && i<12)) {tab[i].setForme(true);}
			if(i<2 || (i>=4 && i<6) || (i>=8 && i<10) || (i>=12 && i<14)) {tab[i].setPleine(true);}
		}
		
		return tab;
	}
	
	/* ---------------------------------------------------- METHODES ---------------------------------------------------------*/
	/*------------------------------------------------------------------------------------------------------------------------*/
	public void setSousMenus(Rectangle m, Color c, int h, int w, float o) {
		m.setFill(c);
		m.setArcHeight(h);
		m.setArcWidth(w);
		m.setOpacity(o);
		
	}
	
	
	public void setImagesBoutonsJetons() {
		String image = "";
		tableauImage = new Image[16];
		
		for(int i=0; i<16 ; i++) {
			image = "Images/jeton2D/"+(i+1)+".png";
			setTableauImage(i,new Image(image));
			setTableauImageView(i,new ImageView(getTableauImage(i)));
			getTableauImageView(i).setFitHeight(65); 
			getTableauImageView(i).setFitWidth(65);
			setTableauBouton(i, new RadioButton());
		}
	}
	
	public void setJetonMenu3D(int fenY) {
		//on met en place les images 3D
		menu3D.setFill(Color.AQUA);
		menu3D.setOpacity(0.2);
		menu3D.setArcHeight(20);
		menu3D.setArcWidth(20);
		
		im3D.setImage(null);
		im3D.setFitHeight(150);
		im3D.setFitWidth(300);
		im3D.setLayoutX(fenY+150);
		im3D.setLayoutY(50+menuJB.getHeight());
		
		Image[] tableauImage3D;
		tableauImage3D = new Image[16];
		String image_3D ="";
		for(int i=0; i<16;i++) {
			image_3D = "Images/jeton3D/"+(i+1)+"_3D.png";
			tableauImage3D[i] = new Image(image_3D);
		}
		
		//on ajoute un evenement à chaque bouton lorsqu'on clique sur celui-ci :
		tableauBouton[0].setOnMouseClicked(e->jetonSelect(fenY, tableauImage3D, 0));
		tableauBouton[1].setOnMouseClicked(e->jetonSelect(fenY, tableauImage3D, 1));
		tableauBouton[2].setOnMouseClicked(e->jetonSelect(fenY, tableauImage3D, 2));
		tableauBouton[3].setOnMouseClicked(e->jetonSelect(fenY, tableauImage3D, 3));
		tableauBouton[4].setOnMouseClicked(e->jetonSelect(fenY, tableauImage3D, 4));
		tableauBouton[5].setOnMouseClicked(e->jetonSelect(fenY, tableauImage3D, 5));
		tableauBouton[6].setOnMouseClicked(e->jetonSelect(fenY, tableauImage3D, 6));
		tableauBouton[7].setOnMouseClicked(e->jetonSelect(fenY, tableauImage3D, 7));
		tableauBouton[8].setOnMouseClicked(e->jetonSelect(fenY, tableauImage3D, 8));
		tableauBouton[9].setOnMouseClicked(e->jetonSelect(fenY, tableauImage3D, 9));
		tableauBouton[10].setOnMouseClicked(e->jetonSelect(fenY, tableauImage3D, 10));
		tableauBouton[11].setOnMouseClicked(e->jetonSelect(fenY, tableauImage3D, 11));
		tableauBouton[12].setOnMouseClicked(e->jetonSelect(fenY, tableauImage3D, 12));
		tableauBouton[13].setOnMouseClicked(e->jetonSelect(fenY, tableauImage3D, 13));
		tableauBouton[14].setOnMouseClicked(e->jetonSelect(fenY, tableauImage3D, 14));
		tableauBouton[15].setOnMouseClicked(e->jetonSelect(fenY, tableauImage3D, 15));
	}
	
	//Positionnement des boutons des jetons
	public void positionnerBoutonJetons(int fenY) {
		for(int i=0; i<16 ; i++) {
			if(tableauDeJeton[i].getCouleur()) {
				if(getTableauDeJeton(i).getForme()) {
					getTableauBouton(i).setLayoutY(20+getTableauBouton(i).getLayoutY()*i);
					getTableauBouton(i).setLayoutX(50+fenY+150*i);
				}
				else {
					getTableauBouton(i).setLayoutY(150+getTableauBouton(i).getLayoutY()*i);
					getTableauBouton(i).setLayoutX(50+fenY+150*(i-4));
				}
				getTableauBouton(i).setGraphic(getTableauImageView(i));
			}
			else {
				setTableauBouton(i, new RadioButton());
				if(getTableauDeJeton(i).getForme()) {
					getTableauBouton(i).setLayoutY(30+getTableauBouton(i).getLayoutY()*i+getMenuJB().getHeight()*2);
					getTableauBouton(i).setLayoutX(50+fenY+150*(i-8));
				}
				else {
					getTableauBouton(i).setLayoutY(150+getTableauBouton(i).getLayoutY()*i+getMenuJB().getHeight()*2);
					getTableauBouton(i).setLayoutX(50+fenY+150*(i-12));
				}
				getTableauBouton(i).setGraphic(getTableauImageView(i));
			}
		
		}
	}
	
	private void afficherImage(int fenY, Image[] tab, int i) {
		affichageMessageBox(FenetrePartie.compteur,fenY);
		im3D.setImage(null);
		im3D.setImage(tab[i]);
	}
	
	public void switchCompteur() {
		switch(FenetrePartie.compteur) {
		case 0:{
			FenetrePartie.compteur=1;
			}break;
		case 2:{
			FenetrePartie.compteur=3;
			}break;
		}
	}
	
	
	public void jetonSelect(int fenY, Image[] tab, int i) {
		afficherImage(fenY, tab, i);
		FenetrePartie.jetonTmp = tableauDeJeton[i];
		
		if (FenetrePartie.compteur == 0 || (FenetrePartie.compteur == 2 && partie.getJ2().getType() == true)) {
			
			if (FenetrePartie.compteur == 0) {
				FenetrePartie.coupTmp = new Coup (coordX, coordY, FenetrePartie.jetonTmp);
				FenetrePartie.compteur = 1;
				affichageMessageBox(FenetrePartie.compteur,fenY);
				FenetrePartie.nbTour++;
			}
			
			if (FenetrePartie.compteur == 2) {
				FenetrePartie.coupTmp = new Coup (coordX, coordY, FenetrePartie.jetonTmp);
				FenetrePartie.compteur = 3;
				affichageMessageBox(FenetrePartie.compteur,fenY);
				FenetrePartie.nbTour++;
			}
			
			//partie.victoireVerif();
			
			if (FenetrePartie.compteur == 1 && partie.getJ2().getType() == false) {
				FenetrePartie.coupTmp = partie.getJ2().jouerTour(FenetrePartie.coupTmp);
				partie.placerJeton(partie.getGrilleJetonsPlacesIJ(FenetrePartie.coupTmp.getX(),FenetrePartie.coupTmp.getY()));
				FenetrePartie.jetonTmp = FenetrePartie.coupTmp.getJetonSelect();
				if(!partie.victoireVerif()) {
					FenetrePartie.compteur = 3;
					affichageMessageBox(FenetrePartie.compteur,fenY);
					FenetrePartie.nbTour++;
					
					int j=0;
					while(j<16 && tableauDeJeton[j] != FenetrePartie.jetonTmp)
						j++;
					
					afficherImage(fenY, tab, j);
				}
				

			}
			
		}
		
	}
	
	public void affichageMessageBox (int x, int fenY) {
		
		for(int i = 0;i<6;i++) {
			msgBox[i].setLayoutX(fenY+100);
			msgBox[i].setLayoutY(730);
			msgBox[i].setTextFill(Color.BLACK);
			msgBox[i].setFont(Font.font("Impact", FenetrePartie.getScene().getHeight() / 40));
		}
		
		
		
		
		switch (x) {
		case 0 :
			removeIm3D();
			msgBox[0].setVisible(true);
			msgBox[1].setVisible(false);
			msgBox[2].setVisible(false);
			msgBox[3].setVisible(false);
			msgBox[4].setVisible(false);
			msgBox[5].setVisible(false);
		break;
		case 1 : 
			msgBox[0].setVisible(false);
			msgBox[1].setVisible(true);
			msgBox[2].setVisible(false);
			msgBox[3].setVisible(false);
			msgBox[4].setVisible(false);
			msgBox[5].setVisible(false);
		break;
		case 2 : 
			msgBox[0].setVisible(false);
			msgBox[1].setVisible(false);
			msgBox[2].setVisible(true);
			msgBox[3].setVisible(false);
			msgBox[4].setVisible(false);
			msgBox[5].setVisible(false);
		break;
		case 3 : 
			msgBox[0].setVisible(false);
			msgBox[1].setVisible(false);
			msgBox[2].setVisible(false);
			msgBox[3].setVisible(true);
			msgBox[4].setVisible(false);
			msgBox[5].setVisible(false);
		break;
		case 4 : 
			msgBox[0].setVisible(false);
			msgBox[1].setVisible(false);
			msgBox[2].setVisible(false);
			msgBox[3].setVisible(false);
			if(FenetrePartie.compteur==3) {
				msgBox[4].setVisible(true);
				msgBox[5].setVisible(false);
			}
			else {
				msgBox[4].setVisible(false);
				msgBox[5].setVisible(true);
			}
		break;
		}
	}

}
